Use C99 uint32_t rather than u_int32_t
[onak.git] / merge.c
diff --git a/merge.c b/merge.c
index 95abbbafbf8d6e663367d34fc68c9ff7e638ccee..9dcbf4d20fc3c784df361cb68e6c6a2d8ec5dac2 100644 (file)
--- a/merge.c
+++ b/merge.c
@@ -1,9 +1,20 @@
 /*
  * merge.c - Routines to merge OpenPGP public keys.
  *
- * Jonathan McDowell <noodles@earth.li>
+ * Copyright 2002-2005,2007,2011 Jonathan McDowell <noodles@earth.li>
  *
- * Copyright 2002-2004 Project Purple
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the Free
+ * Software Foundation; version 2 of the License.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
+ * more details.
+ *
+ * You should have received a copy of the GNU General Public License along with
+ * this program; if not, write to the Free Software Foundation, Inc., 51
+ * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
  *     @a: The first packet to compare.
  *     @b: The second packet to compare.
  *
- *     Takes 2 packets and returns true if they are the same and false
- *     otherwise.
+ *     Takes 2 packets and returns 0 if they are the same, -1 if a is
+ *      less than b, or 1 if a is greater than b.
  */
-bool compare_packets(struct openpgp_packet *a, struct openpgp_packet *b)
+int compare_packets(struct openpgp_packet *a, struct openpgp_packet *b)
 {
-       return (a->tag == b->tag && a->length == b->length &&
-               !memcmp(a->data, b->data, b->length));
+       int ret, len;
+
+       if (a->tag > b->tag) {
+               ret = 1;
+       } else if (b->tag > a->tag) {
+               ret = -1;
+       } else {
+               len = (a->length < b->length) ? a->length : b->length;
+               ret = memcmp(a->data, b->data, len);
+               if (ret == 0 && a->length != b->length) {
+                       ret = (a->length < b->length) ? -1 : 1;
+               }
+       }
+
+       return ret;
 }
 
 /**
@@ -43,7 +67,20 @@ bool compare_packets(struct openpgp_packet *a, struct openpgp_packet *b)
  */
 bool compare_signatures(struct openpgp_packet *a, struct openpgp_packet *b)
 {
-       return (sig_keyid(a) == sig_keyid(b));
+       uint64_t a_keyid, b_keyid;
+       time_t a_creation, b_creation;
+
+       if (a->data[0] != b->data[0]) {
+               /* Different signature versions, so not the same */
+               return false;
+       } else if (a->data[0] == 4 && a->data[1] != b->data[1]) {
+               /* Type 4 signature, but different types */
+               return false;
+       } else {
+               sig_info(a, &a_keyid, &a_creation);
+               sig_info(b, &b_keyid, &b_creation);
+               return (a_creation == b_creation) && (a_keyid == b_keyid);
+       }
 }
 
 /**
@@ -60,7 +97,7 @@ bool find_packet(struct openpgp_packet_list *packet_list,
        bool found = false;
 
        while (!found && packet_list != NULL) {
-               if (compare_packets(packet_list->packet, packet)) {
+               if (compare_packets(packet_list->packet, packet) == 0) {
                        found = true;
                }
                packet_list = packet_list -> next;
@@ -109,7 +146,7 @@ struct openpgp_signedpacket_list *find_signed_packet(
        struct openpgp_signedpacket_list *found = NULL;
 
        while (found == NULL && packet_list != NULL) {
-               if (compare_packets(packet_list->packet, packet)) {
+               if (compare_packets(packet_list->packet, packet) == 0) {
                        found = packet_list;
                }
                packet_list = packet_list -> next;
@@ -136,7 +173,7 @@ bool remove_signed_packet(struct openpgp_signedpacket_list **packet_list,
        bool found = false;
 
        for (cur = *packet_list; !found && (cur != NULL); cur = cur->next) {
-               if (compare_packets(cur->packet, packet)) {
+               if (compare_packets(cur->packet, packet) == 0) {
                        found = true;
                        if (prev == NULL) {
                                *packet_list = cur->next;
@@ -173,7 +210,7 @@ int merge_packet_sigs(struct openpgp_signedpacket_list *old,
        struct openpgp_packet_list      *curpacket = NULL;
        struct openpgp_packet_list      *nextpacket = NULL;
 
-       log_assert(compare_packets(old->packet, new->packet));
+       log_assert(compare_packets(old->packet, new->packet) == 0);
 
        curpacket = new->sigs;
        while (curpacket != NULL) {
@@ -305,12 +342,12 @@ int merge_keys(struct openpgp_publickey *a, struct openpgp_publickey *b)
                /*
                 * Key IDs are the same, so I guess we have to merge them.
                 */
-               curpacket = b->revocations;
+               curpacket = b->sigs;
                while (curpacket != NULL) {
                        nextpacket = curpacket->next;
-                       if (find_packet(a->revocations, curpacket->packet)) {
+                       if (find_packet(a->sigs, curpacket->packet)) {
                                /*
-                                * We already have this revocation, remove it
+                                * We already have this signature, remove it
                                 * from the difference list and free the memory
                                 * allocated for it.
                                 */
@@ -318,8 +355,8 @@ int merge_keys(struct openpgp_publickey *a, struct openpgp_publickey *b)
                                if (lastpacket != NULL) {
                                        lastpacket->next = curpacket->next;
                                } else {
-                                       log_assert(curpacket == b->revocations);
-                                       b->revocations = curpacket->next;
+                                       log_assert(curpacket == b->sigs);
+                                       b->sigs = curpacket->next;
                                }
                                curpacket->next = NULL;
                                free_packet_list(curpacket);
@@ -329,15 +366,15 @@ int merge_keys(struct openpgp_publickey *a, struct openpgp_publickey *b)
                        }
                        curpacket = nextpacket;
                }
-               b->last_revocation = lastpacket;
+               b->last_sig = lastpacket;
 
                /*
* Anything left on b->revocations doesn't exist on
* a->revocations, so add them to the list.
                 */
-               packet_list_add(&a->revocations,
-                               &a->last_revocation,
-                               b->revocations);
+               packet_list_add(&a->sigs,
+                               &a->last_sig,
+                               b->sigs);
 
                /*
                 * Merge uids (signed list).
@@ -350,5 +387,13 @@ int merge_keys(struct openpgp_publickey *a, struct openpgp_publickey *b)
 
        }
 
+       /*
+        * If either key was revoked, make sure both the new ones are marked as
+        * being so.
+        */
+       if (a->revoked || b->revoked) {
+               a->revoked = b->revoked = true;
+       }
+
        return rc;
 }
@@ -329,15 +366,15 @@ int merge_keys(struct openpgp_publickey *a, struct openpgp_publickey *b)
 
                /*
-                * Anything left on b->revocations doesn't exist on
-                * a->revocations, so add them to the list.
+                * Anything left on b->sigs doesn't exist on
+                * a->sigs, so add them to the list.
                 */